-----------------------------------------------------------------------------
--  Generates a function that calculates the skin for the specified nameplate
-----------------------------------------------------------------------------

local format  = string.format
local tinsert = table.insert
local tremove = table.remove
local tconcat = table.concat

local fields  = { 
	reaction  = 'vReaction', 
	type      = 'vType', 
	level     = 'vLevel', 
	target    = 'isTarget', 
	mouseover = 'isHighlighted',
	threat    = 'vThreat',
}

local function NamesToList(names)
	local lines = {}
	local t = { strsplit("\n",names) }
	for i=1,#t do -- Remove comments: any text starting with #@\/-[ characters.
		local s = strtrim( (strsplit( "#@\\\/\-\[", t[i] )) ) -- Don't remove strsplit extra brackets.
		if #s>0 then tinsert( lines, format('["%s"]=true',s) ) end
	end
	return tconcat( lines , ',')
end

local function MakeSkinCheckWithClosure(names, source)
	local lines = { "return function()" }
	for i=1,#names do
		tinsert( lines, format("local names%d = {%s}",i, NamesToList(names[i]) ) )
	end
	tinsert( lines,  source )
	tinsert( lines, 'end' )
    return assert(loadstring(tconcat( lines, "\n")))()()
end

local function CompileSkinCheckFunction(skins, rules)
	wipe(rules)
	local count, handler, lines, names, iff = 0, { 'return function(p)' }, {}, {}, 'if'
	for i=1,#skins do
		local rule = skins[i]
		if #rule>2 and rule[1] ~= '' then 
			for i=3,#rule do
				local c = rule[i]
				if c[1] == 'names' then
					tinsert( names, c[3] )
					tinsert( lines, format("names%d[p.vName]",#names) )
				else
					local typ, line = type(c[3])
					if c[3] == 'nil' then
						line = format( 'p.%s == nil', fields[c[1]] )
					elseif typ == 'string' then
						line = format( 'p.%s %s "%s"', fields[c[1]], c[2], c[3] )
					elseif typ == 'boolean' then
						line = format( 'not p.%s %s %s', fields[c[1]], c[2], tostring(not c[3]) )
					else
						line = format( 'p.%s %s %s', fields[c[1]], c[2], tostring(c[3]) )
					end
					tinsert( lines, line )
				end
				rules[c[1]] = true
			end
			tinsert( handler, format( '%s %s then return "%s"', iff, tconcat(lines,format(' %s ',rule[2])), rule[1] ) )
			iff = "elseif"
			wipe(lines)
			count = count + 1
		end	
	end
	tinsert( handler, format( count>0 and 'else return "%s" end end' or 'return "%s" end', skins[1][1] ) )
	if #names>0 then
		return MakeSkinCheckWithClosure( names, tconcat(handler,"\n") )
	else	
		return assert(loadstring(tconcat(handler,"\n")))()
	end
end

KiwiPlates.CompileSkinCheckFunction = CompileSkinCheckFunction



