
local addon      = KiwiPlates
local widget     = addon:newWidget( 'Target', { 'target' } )
local SendColor  = addon.SendColor
local kvPlates   = addon.kvPlates
local PlatesSetAlpha    = addon.PlatesSetAlpha
local PlateSendToBack   = addon.PlateSendToBack
local PlateBringToFront = addon.PlateBringToFront

local defaultColor = {1,1,1,0.15}

local targetDB
local opaqueNT
local colorEnabled
local kPlateTarget
local ktHighlight, ktEnabled, ktDefault

local function CreateTexture(kPlate, skinName)
	local db = kPlate.db.Target
	if db then
		tex = ktHighlight:CreateTexture(nil, 'OVERLAY')
		tex:SetTexture( db.filename )
		tex:SetVertexColor( unpack(db.color or defaultColor) )
		if db.coords then tex:SetTexCoord( unpack(db.coords) ) end
		tex:SetBlendMode('ADD')
		tex:SetAllPoints()
	else
		tex = ktDefault
	end
	ktHighlight[skinName] = tex
	return tex
end

local function SetHighlight(kPlate)
	local skinName = kPlate.skinName
	local tex = ktHighlight[skinName] or CreateTexture(kPlate, skinName)
	if tex~=ktEnabled then
		if ktEnabled then 
			ktEnabled:Hide() 
		end
		ktEnabled = tex
		ktEnabled:Show()
	end
	ktHighlight:SetParent( kPlate )
	ktHighlight:SetFrameLevel( kPlate:GetFrameLevel() )	
	ktHighlight:SetAllPoints( kPlate )
	ktHighlight:Show()
end

local function Update(kPlate,bPlate)
	if bPlate.isTarget then
		kPlateTarget = kPlate	
		PlateBringToFront( kPlate, bPlate, 'Target', 128 )
		if colorEnabled then
			SendColor.Target( kPlate, targetDB.glowColor )
		end
		if not kPlate.db.Plate.disableHighlight then
			SetHighlight( kPlate )
		end	
	else
		PlateSendToBack( kPlate, bPlate, 'Target', 128 )
		if targetDB.glowEnabled then
			SendColor.Target( kPlate )
		end
	end
end

local function Hide(kPlate)
	if kPlate == kPlateTarget then
		ktHighlight:Hide()
		if colorEnabled then
			SendColor.Target( kPlate )
		end
		kPlateTarget = nil
	end
end

local function TargetChanged( newTarget, oldTarget )
	if opaqueNT and ( not newTarget ~= not oldTarget ) then
		PlatesSetAlpha(newTarget and addon.theme.Plates.alpha or 1)
	end
	if not newTarget then
		ktHighlight:Hide()
	end
end

function widget:OnEnable(addon, theme )
	targetDB     = theme.Target
	opaqueNT     = targetDB.opaqueWhenNoTarget
	colorEnabled = targetDB.colorReceivers and #targetDB.colorReceivers>0
	ktHighlight  = ktHighlight or CreateFrame( 'Frame' )
	ktDefault    = ktDefault or ktHighlight:CreateTexture(nil, 'OVERLAY')
	ktDefault:SetTexture(1,1,1,targetDB.brightness or 0.15)	
	ktDefault:SetBlendMode('ADD')
	ktDefault:SetAllPoints()
	addon.RegisterNotify( 'PlateShow', Update )
	addon.RegisterNotify( 'PlateHide', Hide )	
	addon.RegisterNotify( 'PlateTargetUpdate', Update )
	addon.RegisterNotify( 'TargetChanged', TargetChanged )
	addon.RegisterColorSender( 'Target', targetDB.colorReceivers, targetDB.colorPriority or 5)
	addon.PlatesSetAlpha( addon.TargetGUID and addon.theme.Plates.alpha or 1 )
end

function widget:OnDisable(addon)
	addon.UnregisterNotify( 'PlateShow', Update )
	addon.UnregisterNotify( 'PlateHide', Hide )		
	addon.UnregisterNotify( 'PlateTargetUpdate', Update )
	addon.UnregisterNotify( 'TargetChanged', TargetChanged )
	addon.UnregisterColorSender( 'Target')
end
