
local addon  = KiwiPlates
local status = addon:newStatus( 'auras' )
local Notify = addon.Notify
local bPlateOfGuid = addon.bPlateOfGuid

local next = next
local rawget = rawget
local select = select
local UnitGUID = UnitGUID
local UnitAura = UnitAura
local GetSpellInfo = GetSpellInfo
local UnitIsFriend = UnitIsFriend

local playerGUID
local AURA_EVENTS = { 	SPELL_AURA_REMOVED = 0 , SPELL_AURA_BROKEN = 0, SPELL_AURA_BROKEN_SPELL = 0, 
					    SPELL_AURA_APPLIED = 1,  SPELL_AURA_REFRESH = 1, SPELL_AURA_APPLIED_DOSE = 1, SPELL_AURA_REMOVED_DOSE = 1, }
local auraExpiration = setmetatable( {}, { __index = function(t,k) local v = {}; t[k] = v; return v end } )
local auraCount      = setmetatable( {}, { __index = function(t,k) local v = {}; t[k] = v; return v end } )
local auraIcon       = setmetatable( {}, { __index = function(t,k) local _,_,icon = GetSpellInfo(k); t[k] = icon; return icon end } )
local auraName       = setmetatable( {}, { __index = function(t,k) local name = GetSpellInfo(k); t[k] = name; return name end } )

local debuffsMine = {}
local debuffsMineIsBlackList = true

--function(_, timestamp, event, _, srcGUID, srcNAME, _,_, dstGUID, dstName, _,_, spellId, spellId )
local function CombatLogEvent(_,timestamp, event, _, srcGUID, ... )
	if srcGUID == playerGUID then
		local action = AURA_EVENTS[event]
		if action then
			local spellName = select( 9, ...)
			local filter = debuffsMine[spellName]
			if not filter == debuffsMineIsBlackList then
				local spellId = select( 8, ...)
				local dstGUID = select(4, ...)
				if action==1 then
					local bPlate, count, expires, _ = bPlateOfGuid[dstGUID] 
					if bPlate and bPlate.isTarget then
						_,_,_,count,_,_,expires = UnitAura('target', spellName, nil, 'HARMFUL|PLAYER' )
					end	
					if not expires then	count, expires = 0, 0 end
					auraExpiration[dstGUID][spellId] = expires 
					auraCount[dstGUID][spellId]      = count
					if bPlate then
						local kPlate = bPlate.kPlate
						if kPlate and kPlate.Auras then
							Notify.PlateAuraUpdate( kPlate, spellId, expires, count )							
						end
					end	
				else
					local expiration = auraExpiration[dstGUID][spellId]
					if expiration then
						auraExpiration[dstGUID][spellId] = nil
						auraCount[dstGUID][spellId] = nil
						local bPlate = bPlateOfGuid[dstGUID] 
						if bPlate then
							Notify.PlateAuraRemove( bPlate.kPlate, spellId )
						end	
					end	
				end
			end	
		end
	end	
end

local toremove = {}
local function ParseAuras(unit)
	local guid = UnitGUID(unit)
	if guid then
		local expirations = rawget( auraExpiration, guid )
		if expirations then
			local counts = auraCount[guid]
			local kPlate = bPlateOfGuid[guid]
			kPlate = kPlate and kPlate.kPlate
			for auraId,expiration in next, expirations do
				local spellName = auraName[auraId]
				local _,_,_,count,_,_,expires = UnitAura( unit, spellName, nil, 'HARMFUL|PLAYER' )
				if expires then
					count = count or 1
					if expires ~= expirations[auraId] or count ~= counts[auraId] then
						expirations[auraId]  = expires
						counts[auraId]       = count
						if kPlate then Notify.PlateAuraUpdate( kPlate, auraId, expires, count ) end	
					end	
				else
					toremove[#toremove+1] = auraId
					if kPlate then Notify.PlateAuraRemove( kPlate, auraId ) end
				end
			end
			if #toremove>0 then
				for i=1,#toremove do
					expirations[ toremove[i] ] = nil
				end
				wipe(toremove)
			end
		end	
	end	
end

local function PlateMouseEnter()
	ParseAuras('mouseover')
end

local function TargetChanged(guid)
	if guid then
		ParseAuras('target')
	end
end

local function PlateGuidStored(bPlate)
	local kPlate = bPlate.kPlate
	if kPlate then Notify.PlateAurasUpdate( kPlate, bPlate ) end
end

function status:OnEnable(addon)
	playerGUID = UnitGUID('player')
	local _,playerClass = UnitClass('player')
	local classdb = addon.db.auras[playerClass] or {}
	debuffsMineIsBlackList = not classdb.debuffsMineIsWhiteList
	local t = { strsplit("\n", classdb.debuffsMine or '') }
	for i=1,#t do
		debuffsMine[ strtrim(t[i]) ] = 'HARMFUL|PLAYER' 
	end
	--
	addon.RegisterNotify( 'PlateMouseEnter', PlateMouseEnter )
	addon.RegisterNotify( 'PlateGuidStored', PlateGuidStored )	
	addon.RegisterNotify( 'TargetChanged',  TargetChanged)
	addon:RegisterEvent( 'COMBAT_LOG_EVENT_UNFILTERED', CombatLogEvent )
end

function status:OnDisable(addon)
	wipe(auraExpiration)
	wipe(auraCount)
	addon.UnregisterNotify( 'PlateMouseEnter',  PlateMouseEnter )
	addon.UnregisterNotify( 'PlateGuidStored', PlateGuidStored )
	addon.UnregisterNotify( 'TargetChanged',  TargetChanged)
	addon:UnregisterEvent('COMBAT_LOG_EVENT_UNFILTERED')
end

--- Publish some usefull stuff

addon.auraExpiration = auraExpiration
addon.auraCount      = auraCount
addon.auraIcon       = auraIcon
addon.auraName       = auraName