--[[ 
Plates Mouse Enter/Leave detection status
Unfixed glitch: Mouse enter on target nameplate is not detected because UPDATE_MOUSEOVER_UNIT is not fired for target.
--]]

local addon  = KiwiPlates
local status = addon:newStatus( 'mouseover' )
local Notify = addon.Notify
local PlateSkinRules = addon.PlateSkinRules

local next = next
local C_Timer = C_Timer
local UnitGUID = UnitGUID
local WorldFrame = WorldFrame
local GetMouseFocus = GetMouseFocus
local UnitHealthMax = UnitHealthMax

local bPlateMouseOver
local bPlateOfGuid  = {}
local unknowPlates = {} -- Plates with no guid

local function StoreGUID(guid, bPlate, unit)
	bPlate.guid = guid
	bPlateOfGuid[ guid ] = bPlate
	unknowPlates[ bPlate ] = nil
	bPlate.vMaxHealth = UnitHealthMax(unit)	
	Notify.PlateGuidStored( bPlate, guid, unit )
end

local function OnMouseLeave()
	if bPlateMouseOver then
		bPlateMouseOver.isHighlighted = false
		local kPlate = bPlateMouseOver.kPlate
		if kPlate then
			Notify.PlateMouseLeave( kPlate, bPlateMouseOver )
			if PlateSkinRules.mouseover then addon.ReSkinPlate(bPlateMouseOver) end	
		end	
		bPlateMouseOver = nil
	end	
end

-- Using a timer to check mouse leave because UPDATE_MOUSEOVER_UNIT is only fired for mouse enter
local function CheckMouseLeave()
	if bPlateMouseOver then
		local guid = UnitGUID('mouseover')
		if (not guid and not bPlateMouseOver:IsMouseOver()) or (guid and bPlateOfGuid[guid]~=bPlateMouseOver) then
			OnMouseLeave()
		else
			C_Timer.After(0.1,CheckMouseLeave)
		end
	end	
end

local function OnMouseEnter(bPlate)
	if bPlate~=bPlateMouseOver then
		OnMouseLeave()
		bPlateMouseOver = bPlate
		bPlate.isHighlighted = true    
		local kPlate = bPlate.kPlate
		if kPlate then
			if PlateSkinRules.mouseover then addon.ReSkinPlate(bPlate) end
			Notify.PlateMouseEnter( kPlate, bPlate )
		end	
		C_Timer.After(0.1,CheckMouseLeave)
	end
	-- Trying to locate target if target was not detected, see statuses/target.lua
	if addon.TargetGUID and (not addon.GetTargetPlate()) and bPlate:GetAlpha() > 0.9 then
		addon.SetTargetPlate(bPlate)
	end
end

local function UpdateMouseOverUnit()
	if GetMouseFocus() == WorldFrame then
		local guid = UnitGUID('mouseover')
		if guid then
			local bPlate = bPlateOfGuid[guid]
			if bPlate then
				OnMouseEnter(bPlate)
				return
			end
			for bPlate in next,unknowPlates do
				if bPlate.bHighlight:IsShown() then
					StoreGUID( guid, bPlate, 'mouseover' )
					OnMouseEnter(bPlate)
					return
				end
			end
		end	
	else
		OnMouseLeave()
	end	
end

local function PlateShow(bPlate)
	unknowPlates[bPlate] = true
end

local function PlateHide(bPlate)
	local guid = bPlate.guid
	if guid then
		bPlateOfGuid[ guid ] = nil	
		bPlate.guid = nil
		bPlate.vMaxHealth = nil
		bPlate.isHighlighted = nil
		if bPlate==bPlateMouseOver then
			bPlateMouseOver = nil
		end	
		Notify.PlateGuidRemoved( bPlate, guid )		
	else
		unknowPlates[ bPlate ] = nil
	end
end

function status:OnEnable(addon, dbx)
	addon.RegisterNotify( 'bPlateShow', PlateShow )
	addon.RegisterNotify( 'bPlateHide', PlateHide )
	addon:RegisterEvent( 'UPDATE_MOUSEOVER_UNIT', UpdateMouseOverUnit )
end

function status:OnDisable(addon)
	addon.UnregisterNotify( 'bPlateShow', PlateShow )
	addon.UnregisterNotify( 'bPlateHide', PlateHide )
	addon:UnregisterEvent( 'UPDATE_MOUSEOVER_UNIT' )
end

--- Public objects
addon.StoreGUID = StoreGUID
addon.bPlateOfGuid = bPlateOfGuid

