
local addon   = KiwiPlates
local Options = KiwiPlatesOptions

local options = {}

local skins_values = {}
local convert = { ['true'] = true, ['false'] = false }
local keys = { type = 'type', reaction = 'reaction', level = 'level', target = 'target' , mouseover = 'mouseover', names = 'names', threat = 'threat' }
local def_values = { type = 'elite' , reaction = 'hostile', level = 100, target = true , mouseover = true, names = "", threat = "tanked" }
local values = {
    type     = { boss = 'boss', elite = 'elite', rare = 'rare', normal = 'normal', player = 'player', trivial = 'trivial', z = '|cFFff0000delete|r' },
	reaction = { hostile = 'hostile', neutral = 'neutral', friendly = 'friendly', tapped = 'tapped', z = '|cFFff0000delete|r' },
	level     = { ['<90'] = '<90', ['<95'] = '<95', ['<100'] = '<100', ['100'] = '100', ['101'] = '101', ['102'] = '102', z = '|cFFff0000delete|r' },
	target    = { ['true'] = 'true', ['false'] = 'false', z = '|cFFff0000delete|r' },
	mouseover = { ['true'] = 'true', ['false'] = 'false', z = '|cFFff0000delete|r' },
	threat    = { tanked = 'tanked', aggroed = 'aggroed', offtanked = 'offtanked', ['nil'] = 'none', z = '|cFFff0000delete|r' }
}

local function MakeOptions(rootdb)

	local function MakeItem(db, order)
		local ocond
		local function MakeCondition(key, index)
			local cond = {
				name = key,
				get = function(info) 
					local comp  = db[index][2]
					local value = db[index][3]
					return ( comp == '==' ) and tostring(value) or  (comp .. tostring(value))
				end,
				set = function(info, value)	
					if value == 'z' or (key == 'names' and strtrim(value)=='') then
						table.remove( db, index )
						MakeItem(db, order)
					else
						local comp, val = '==', tonumber(value) or convert[value]
						if not val then
							local ncomp = strsub(value,1,1) 
							if ncomp == '<' or ncomp == '>' then
								val = tonumber(strsub(value,2))
								if val then comp = ncomp end
							end
						end
						db[index][2] = comp
						db[index][3] = val or value
					end	
					addon:MakeSkinCheckFunction()
					addon.ReSkinPlates()
					info.handler:OnChange()
				end,
				confirm = function(info, value)
					if value == 'z' then 
						return 'Are you sure you want to remove this condition ?' 
					elseif key=='names' and strtrim(value)=='' then
						return 'Names field is empty. Do you want to remove this condition ?'
					end
				end,
			}	
			if key == 'names' then
				cond.type = 'input'
				cond.multiline = 10
				cond.width = 'full'
				cond.values = nil
				cond.order = 99
				ocond[key] = cond
			else
				cond.type = 'select'
				cond.width = 'half'
				cond.order = index
				cond.values = values[key] or {}
				ocond[key..index] = cond
			end
		end
		if order > 1 then
			ocond = {}
			options['skin'..order] = { type = 'group', order = order+100, name = 'Skin '.. order, args = ocond }
		else
			ocond = options
			for k,v in next,options do
				if v.order<100 then options[k] = nil end
			end
		end
		ocond.header = { type = 'header', order = 0, name = (order==1 and 'Default Skin' or 'Skin '.. order), }					
		ocond.skin = {
			type = 'select', width = 'full', order = 1,
			name = '',
			get = function() return db[1] end,
			set = function(info, value)
				db[1] = value
				addon:EnableWidgets( addon.skins[value] )
				addon:MakeSkinCheckFunction()
				addon.ReSkinPlates()
				info.handler:OnChange()				
			end,
			values = function()
				wipe(skins_values)
				for name in pairs(addon.skins) do
					skins_values[name] = name
				end
				return skins_values
			end,
		} 
		for i=3,#db do
			MakeCondition( db[i][1], i )
		end
		if db[2] then
			ocond.headercond1 = { type = 'header', order = 1.1, name = order==1 and 'Additional Activation Rules' or 'Activation Rules' }
			ocond.headercond2 = { type = 'header', order = 1.9, name = 'Conditions' }
			ocond.AndOr = {
				type = 'select', width = 'normal', order = 1.5,
				name = 'Required for activation',
				desc = '',
				get = function() return db[2] end,
				set = function(info,key)
					db[2] = key
					addon:MakeSkinCheckFunction()
					addon.ReSkinPlates()
					info.handler:OnChange()
				end,
				values = { ['and'] = 'All conditions', ['or'] = 'Any condition' },
			}
			ocond.toggle = {
				type = 'select', width = 'normal', order = 1.6,
				name = 'Add a New Condition',
				desc = 'Select a condition variable to add.',
				get = function() end,
				set = function(info,key)
					if key~='names' or not ocond[key] then
						db[#db+1] = { key, '==', def_values[key] }	
						MakeCondition( key, #db )
						addon:MakeSkinCheckFunction()
						addon.ReSkinPlates()
						info.handler:OnChange()
					end	
				end,
				values = keys,
			}
			ocond.filler = {
				type = 'select', width = 'half', order = 1.7,
				name = 'Actions',
				desc = '',
				get = function() end,
				set = function(info,key) 
					if key==1 then
						rootdb[order], rootdb[order-1] = rootdb[order-1], rootdb[order]
					elseif key==2 then
						rootdb[order], rootdb[order+1] = rootdb[order+1], rootdb[order]
					elseif key==3 then
						if order>1 then
							table.remove( rootdb, order )
						else
							local skinName = db[1]
							wipe(db)
							db[1] = skinName
						end
					end
					MakeOptions(rootdb)
					addon:MakeSkinCheckFunction()
					addon.ReSkinPlates()
					info.handler:OnChange()
				end,
				values = function()
					local values = { [3] = '|cFFFF0000DELETE|r' }
					if order>1 then
						if order>2       then values[1] = '|cFF00FF00UP|r'   end
						if order<#rootdb then values[2] = '|cFF00FF00DOWN|r' end
					end	
					return values
				end,
				confirm = function(info, key) 
					if key == 3 then 
						if order == 1 then
							return 'Are you sure you want to remove the additional activation rules ?'
						else
							return 'Are you sure you want to remove this Skin and all activation rules and conditions ?' 
						end	
					end 
				end
			}
		else
			ocond.headeradd = { type = 'header', order = 1.4, name = '' }
			ocond.addconditions = {
				type = "execute",
				order = 1.5,
				width = "full",
				name = "Enable additional activation conditions",
				func = function(info) 
					db[2] = 'and'
					MakeItem( db, order ) 
					info.handler:OnChange()
				end,
			}		
		end	
	end

	wipe(options)
	for index,itemdb in ipairs(rootdb) do
		MakeItem( itemdb, index )
	end	
	options.bottom = { type = 'header', order = 150, name = '', }
	options.newskin = {
		type = "execute",
		order = 160,
		width = "full",
		name = "Add another Skin",
		func = function(info) 
			local index = #rootdb+1
			rootdb[index] = { '', 'and' }
			MakeItem( rootdb[index], index ) 
			info.handler:OnChange()
		end,
	}
	return options

end 

KiwiPlatesOptions:RegisterModuleOptions( 'Skins', MakeOptions , { order = 0 } )
